var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as S3 from 'aws-sdk/clients/s3';
export class CloudStorage {
    constructor(config) {
        this.config = config;
        this.client = new S3(config.storage);
    }
    static init(config) {
        return new CloudStorage(config);
    }
    head(path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const params = {
                    Bucket: this.config.database,
                    Key: path
                };
                const data = yield this.client.headObject(params).promise();
                return data;
            }
            catch (err) {
                throw err;
            }
        });
    }
    set(value, path, 
    // tslint:disable-next-line:no-any
    metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const params = {
                    Body: JSON.stringify(value),
                    Bucket: this.config.database,
                    Key: path,
                    Metadata: metadata
                };
                const data = yield this.client.putObject(params).promise();
                return data;
            }
            catch (err) {
                throw err;
            }
        });
    }
    get(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                Bucket: this.config.database,
                Key: path
            };
            // tslint:disable-next-line:no-any
            let data;
            try {
                data = yield this.client.getObject(params).promise();
                data.Body = JSON.parse(data.Body.toString('utf8'));
                // if (data.Body && data.Body.byteLength > 1) {
                //   data.Body = JSON.parse(data.Body.toString('utf8'))
                // }
                return data;
            }
            catch (err) {
                throw err;
            }
        });
    }
    remove(path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const params = {
                    Bucket: this.config.database,
                    Key: path
                };
                const data = yield this.client.deleteObject(params).promise();
                return data;
            }
            catch (err) {
                throw err;
            }
        });
    }
    // tslint:disable-next-line:no-any
    copy(path, eTag, metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                Bucket: this.config.database,
                Key: path,
                CopySource: this.config.database + '/' + path,
                CopySourceIfMatch: eTag,
                MetadataDirective: 'REPLACE',
                Metadata: metadata
            };
            try {
                const data = yield this.client.copyObject(params).promise();
                return data;
            }
            catch (err) {
                throw err;
            }
        });
    }
}
//# sourceMappingURL=cloud.js.map